@@ -1,7 +1,7 @@
 #
 # -*- coding: iso-8859-1 -*-
 #
-# Copyright (C) 2006, 2007, 2008, 2009 Loic Dachary <loic@dachary.org>
+# Copyright (C) 2006 - 2010 Loic Dachary <loic@dachary.org>
 # Copyright (C)       2008, 2009 Bradley M. Kuhn <bkuhn@ebb.org>
 # Copyright (C)             2008 Johan Euphrosine <proppy@aminche.com>
 # Copyright (C) 2004, 2005, 2006 Mekensleep
@@ -42,6 +42,7 @@ from pokerengine import pokergame
 from pokernetwork.user import User, checkNameAndPassword
 from pokernetwork.pokerpackets import *
 from pokernetwork.pokerexplain import PokerExplain
+from pokernetwork.pokerrestclient import PokerRestClient
 from twisted.internet import protocol, reactor, defer
 
 DEFAULT_PLAYER_USER_DATA = { 'ready': True }
@@ -62,7 +63,16 @@ class PokerAvatar:
         self.has_session = False
         self.bugous_processing_hand = False
         self.noqueuePackets()
+        self._block_longpoll_deferred = False
+        self._longpoll_deferred = None
+        self.game_id2rest_client = {}
+        self.distributed_args = '?explain=no'
+        self.longPollTimer = None
+        self._flush_next_longpoll = False
 
+    def setDistributedArgs(self, uid, auth):
+        self.distributed_args = '?explain=no&uid=%s&auth=%s' % ( uid, auth )
+        
     def __str__(self):
         return "PokerAvatar serial = %s, name = %s" % ( self.getSerial(), self.getName() )
 
@@ -131,9 +141,7 @@ class PokerAvatar:
 
         if self.explain:
             self.explain.handleSerial(PacketSerial(serial = serial))
-        if self.service.serial2client.has_key(serial):
-            self.service.destroyAvatar(self.service.serial2client[serial])
-        self.service.serial2client[serial] = self
+        self.service.avatar_collection.add(serial, self)
         self.tourneyUpdates(serial)
         self.loginTableUpdates(serial)
     
@@ -150,7 +158,7 @@ class PokerAvatar:
 
         self.sendPacketVerbose(PacketSerial(serial = self.user.serial))
         if PacketPokerRoles.PLAY in self.roles:
-            self.service.serial2client[serial] = self
+            self.service.avatar_collection.add(serial, self)
         if self.service.verbose:
             self.message("user %s/%d logged in" % ( self.user.name, self.user.serial ))
         if self.protocol:
@@ -193,7 +201,7 @@ class PokerAvatar:
     def logout(self):
         if self.user.serial:
             if PacketPokerRoles.PLAY in self.roles:
-                del self.service.serial2client[self.user.serial]
+                self.service.avatar_collection.remove(self.user.serial, self)
             if self.has_session:
                 self.service.sessionEnd(self.getSerial())
             self.user.logout()
@@ -249,6 +257,7 @@ class PokerAvatar:
         # This method was introduced when we added the force-disconnect as
         # the stop-gap.
         self._packets_queue.extend(newPackets)
+        self.flushLongPollDeferred()
         warnVal = int(.75 * self.service.getClientQueuedPacketMax())
         if len(self._packets_queue) >= warnVal:
             # If we have not warned yet that packet queue is getting long, warn now.
@@ -290,10 +299,22 @@ class PokerAvatar:
         #    file.  However, should we have them later, we'd obviously
         #    want those strings to be localized for the client, at least
         #    during packet sending.
+        #
+        #    Note that _ default value depends of locale installation
+        #    by pokerservice, as
+        #    http://docs.python.org/library/gettext.html point out,
+        #    gettext.install installs the function _() in Python’s
+        #    builtins namespace. Assigning it to self.localeFunc
+        #    convert it to a global that is file wise (as pointed
+        #    above).
 
         global _
         if self.localeFunc:
-            avatarSavedUnder = _
+            # First, if our _() has never been defined, we simply set it to None
+            try:
+                self._avatarSavedUnder = _
+            except NameError:
+                self._avatarSavedUnder = None
             _ = self.localeFunc
             pokergameSavedUnder = pokergame_init_i18n('', self.localeFunc)
 	if self.explain and not isinstance(packet, defer.Deferred) and packet.type != PACKET_ERROR:
@@ -318,7 +339,7 @@ class PokerAvatar:
 	    for packet in packets:
                 self.protocol.sendPacket(packet)
         if self.localeFunc:
-            _ = avatarSavedUnder
+            _ = self._avatarSavedUnder
             pokergame_init_i18n('', pokergameSavedUnder)
 
     # Below, we assign the method queueDeferred() is the same as
@@ -341,8 +362,122 @@ class PokerAvatar:
         else:
             return False
 
+    def longpollDeferred(self):
+        self._longpoll_deferred = defer.Deferred()
+        d = self.flushLongPollDeferred()
+        if not d.called:
+            def longPollDeferredTimeout():
+                self.longPollTimer = None
+                self._longpoll_deferred = None
+                packets = self.resetPacketsQueue()
+                if self.service.verbose > 3:
+                    self.message("longPollDeferredTimeout(%s): " % str(packets))
+                d.callback(packets)
+            self.longPollTimer = reactor.callLater(self.service.long_poll_timeout, longPollDeferredTimeout)
+        return d
+
+    def blockLongPollDeferred(self):
+        self._block_longpoll_deferred = True
+        
+    def unblockLongPollDeferred(self):
+        self._block_longpoll_deferred = False
+        self.flushLongPollDeferred()
+
+    def flushLongPollDeferred(self):
+        if self._block_longpoll_deferred == False and self._longpoll_deferred and (len(self._packets_queue) > 0 or self._flush_next_longpoll):
+            self._flush_next_longpoll = False
+            packets = self.resetPacketsQueue()
+            if self.service.verbose > 3:
+                self.message("flushLongPollDeferred(%s): " % str(packets))
+            d = self._longpoll_deferred
+            self._longpoll_deferred = None
+            d.callback(packets)
+            if self.longPollTimer and self.longPollTimer.active():
+                self.longPollTimer.cancel()
+            return d
+        return self._longpoll_deferred
+
+    def longPollReturn(self):
+        if self._longpoll_deferred:
+            packets = self.resetPacketsQueue()
+            if self.service.verbose > 3:
+                self.message("longPollReturn(%s): " % str(packets))
+            d = self._longpoll_deferred
+            self._longpoll_deferred = None
+            d.callback(packets)
+            if self.longPollTimer and self.longPollTimer.active():
+                self.longPollTimer.cancel()
+        else:
+            self._flush_next_longpoll = True
+            
+    def handleDistributedPacket(self, request, packet, data):
+        resthost, game_id = self.service.packet2resthost(packet)
+        if resthost:
+            return self.distributePacket(packet, data, resthost, game_id)
+        else:
+            return self.handlePacketDefer(packet)
+
+    def getOrCreateRestClient(self, resthost, game_id):
+        #
+        # no game_id means the request must be delegated for tournament
+        # registration or creation. Not for table interaction.
+        #
+        ( host, port, path ) = resthost
+        path += self.distributed_args
+        if self.service.verbose > 3:
+            self.message("getOrCreateRestClient(%s, %d, %s, %s)" % ( host, port, path, str(game_id) ))
+        if game_id:
+            if not self.game_id2rest_client.has_key(game_id):
+                if self.service.verbose > 1:
+                    self.message("getOrCreateRestClient(%s, %d, %s, %s): create" % ( host, port, path, str(game_id) ))
+                self.game_id2rest_client[game_id] = PokerRestClient(host, port, path, longPollCallback = lambda packets: self.incomingDistributedPackets(packets, game_id), verbose = self.service.verbose)
+            client = self.game_id2rest_client[game_id]
+        else:
+            client = PokerRestClient(host, port, path, longPollCallback = None, verbose = self.service.verbose)
+        return client
+            
+    def distributePacket(self, packet, data, resthost, game_id):
+        ( host, port, path ) = resthost
+        client = self.getOrCreateRestClient(resthost, game_id)
+        d = client.sendPacket(packet, data)
+        d.addCallback(lambda packets: self.incomingDistributedPackets(packets, game_id))
+        d.addCallback(lambda x: self.resetPacketsQueue())
+        return d
+            
+    def incomingDistributedPackets(self, packets, game_id):
+        if self.service.verbose > 3:
+            self.message("incomingDistributedPackets(%s, %s)" % ( str(packets), str(game_id) ))
+        self.blockLongPollDeferred()
+        for packet in packets:
+            self.sendPacket(packet)
+        self.unblockLongPollDeferred()
+        if game_id:
+            if game_id not in self.tables and (not(self.explain) or not(self.explain.games.gameExists(game_id))):
+                #
+                # discard client if nothing pending and not in the list
+                # of active tables
+                #
+                client = self.game_id2rest_client[game_id]
+                if ( len(client.queue.callbacks) <= 0 or
+                     client.pendingLongPoll ):
+                    if self.service.verbose > 1:
+                        self.message("incomingDistributedPackets: del %d" % game_id)
+                    self.game_id2rest_client[game_id].clearTimeout()
+                    del self.game_id2rest_client[game_id]
+
     def handlePacketDefer(self, packet):
+        if self.service.verbose > 2:
+            self.message("handlePacketDefer(%d): " % self.getSerial() + str(packet))
+
         self.queuePackets()
+
+        if packet.type == PACKET_POKER_LONG_POLL:
+            return self.longpollDeferred()
+
+        if packet.type == PACKET_POKER_LONG_POLL_RETURN:
+            self.longPollReturn()
+            return []
+
         self.handlePacketLogic(packet)
         packets = self.resetPacketsQueue()
         if len(packets) == 1 and isinstance(packets[0], defer.Deferred):
@@ -369,7 +504,7 @@ class PokerAvatar:
     def handlePacketLogic(self, packet):
         if self.service.verbose > 2 and packet.type != PACKET_PING:
             self.message("handlePacketLogic(%d): " % self.getSerial() + str(packet))
-        
+
         if packet.type == PACKET_POKER_EXPLAIN:
             if self.setExplain(packet.value):
                 self.sendPacketVerbose(PacketAck())
@@ -396,11 +531,6 @@ class PokerAvatar:
         if packet.type == PACKET_PING:
             return
         
-        if packet.type == PACKET_POKER_POLL:
-            if packet.tourney_serial != 0 and not (packet.tourney_serial in self.tourneys):
-                self.sendPacketVerbose(PacketPokerTourneyFinish(tourney_serial = packet.tourney_serial))
-            return
-        
         if not self.isAuthorized(packet.type):
             self.sendPacketVerbose(PacketAuthRequest())
             return
@@ -597,10 +727,7 @@ class PokerAvatar:
                 self.message("packet for table " + str(table.game.id))
             game = table.game
 
-            if packet.type == PACKET_POKER_GET_PLAYER_STATS:
-                self.sendPacketVerbose(self.getPlayerStats(table, packet.serial))
-
-            elif packet.type == PACKET_POKER_READY_TO_PLAY:
+            if packet.type == PACKET_POKER_READY_TO_PLAY:
                 if self.getSerial() == packet.serial:
                     self.sendPacketVerbose(table.readyToPlay(packet.serial))
                 else:
@@ -943,6 +1070,8 @@ class PokerAvatar:
                             self.performPacketPokerSit(
                                PacketPokerSit(serial = mySerial, game_id = table.game.id),
                                table)
+                            table.update()
+
     # -------------------------------------------------------------------------
     def setPlayerInfo(self, packet):
         self.user.url = packet.url
@@ -978,14 +1107,6 @@ class PokerAvatar:
                                message = "Not logged in",
                                other_type = PACKET_POKER_GET_PLAYER_INFO)
     
-    def getPlayerStats(self, table, serialSought):
-        if self.user.isLogged():
-            return self.service.getUserStatsLookup().getAttrsAsPacket(table = table, serial = serialSought)
-        else:
-            return PacketError(code = PacketPokerGetPlayerStats.NOT_LOGGED,
-                               message = "Not logged in",
-                               other_type = PACKET_POKER_GET_PLAYER_STATS)
-
     def listPlayers(self, packet):
         table = self.service.getTable(packet.game_id)
         if table:
@@ -1096,9 +1217,12 @@ class PokerAvatar:
                                                            auto = player.auto,
                                                            auto_blind_ante = player.auto_blind_ante,
                                                            wait_for = player.wait_for,
-                                                           seat = player.seat))
-            # Send the stats packet, generated by the UserStats package.
-            self.sendPacketVerbose(self.service.getUserStatsLookup().getAttrsAsPacket(table = table, serial = player.serial))
+                                                           seat = player.seat,
+                                                           buy_in_payed = player.buy_in_payed))
+            if self.service.has_ladder:
+                packet = self.service.getLadder(game.id, table.currency_serial, player.serial)
+                if packet.type == PACKET_POKER_PLAYER_STATS:
+                    self.sendPacketVerbose(packet)
             if not game.isPlaying(player.serial):
                 self.sendPacketVerbose(PacketPokerPlayerChips(game_id = game.id,
                                                               serial = player.serial,
@@ -1141,6 +1265,10 @@ class PokerAvatar:
         for table in self.tables.values():
             table.disconnectPlayer(self, self.getSerial())
         self.logout()
+        for key, restclient in self.game_id2rest_client.iteritems():
+            restclient.clearTimeout()
+            restclient.longPollFrequency = -1
+        self.game_id2rest_client = {}
 
     def getUserInfo(self, serial):
         self.service.autorefill(serial)
@@ -1154,7 +1282,9 @@ class PokerAvatar:
         game = table.game
         player = game.getPlayer(serial)
         seat = player and player.seat
-        if game.removePlayer(serial):
+        avatars = table.avatar_collection.get(serial)
+        self_is_last_avatar = len(avatars) == 1 and avatars[0] == self
+        if self_is_last_avatar and game.removePlayer(serial):
             #
             # If the player is not in a game, the removal will be effective
             # immediately and can be announced to all players, including
