@@ -1,7 +1,7 @@
 #
 # -*- py-indent-offset: 4; coding: iso-8859-1 -*-
 #
-# Copyright (C) 2006, 2007, 2008 Loic Dachary <loic@dachary.org>
+# Copyright (C) 2006, 2007, 2008, 2009 Loic Dachary <loic@dachary.org>
 # Copyright (C) 2008             Bradley M. Kuhn <bkuhn@ebb.org>
 # Copyright (C) 2006 Mekensleep <licensing@mekensleep.com>
 #                    24 rue vieille du temple, 75004 Paris
@@ -34,6 +34,8 @@ from pokernetwork import currencyclient
 from pokernetwork import pokerlock
 from pokernetwork.pokerpackets import *
 
+PokerLock = pokerlock.PokerLock
+
 class PokerCashier:
 
     def __init__(self, settings):
@@ -42,7 +44,7 @@ class PokerCashier:
         self.currency_client = currencyclient.CurrencyClient()
         self.parameters = settings.headerGetProperties("/server/cashier")[0]
         if self.parameters.has_key('pokerlock_queue_timeout'):
-            pokerlock.PokerLock.queue_timeout = int(self.parameters['pokerlock_queue_timeout'])
+            PokerLock.queue_timeout = int(self.parameters['pokerlock_queue_timeout'])
         self.db = None
         self.db_parameters = settings.headerGetProperties("/server/database")[0]
         self.locks = {}
@@ -414,7 +416,7 @@ class PokerCashier:
         else:
             create_lock = True
         if create_lock:
-            self.locks[name] = pokerlock.PokerLock(self.db_parameters)
+            self.locks[name] = PokerLock(self.db_parameters)
             self.locks[name].verbose = self.verbose
             self.locks[name].start()
 
