@@ -1,6 +1,6 @@
 # -*- py-indent-offset: 4; coding: iso-8859-1; mode: python -*-
 #
-# Copyright (C) 2007, 2008 Loic Dachary <loic@dachary.org>
+# Copyright (C) 2007, 2008, 2009 Loic Dachary <loic@dachary.org>
 #
 # This software's license gives you freedom; you can copy, convey,
 # propagate, redistribute and/or modify this program under the terms of
@@ -357,6 +357,8 @@ class PokerExplain:
             elif ( packet.type == PACKET_POKER_PLAYER_LEAVE or
                    packet.type == PACKET_POKER_TABLE_MOVE ) :
                 game.removePlayer(packet.serial)
+                if packet.serial == self.getSerial():
+                    self.games.deleteGame(game.id)
                 if packet.type == PACKET_POKER_TABLE_MOVE:
                     forward_packets.append(PacketPokerPlayerLeave(game_id = packet.game_id,
                                                                   serial = packet.serial,
