@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006, 2007, 2008 Loic Dachary <loic@dachary.org>
+# Copyright (C) 2006, 2007, 2008, 2009 Loic Dachary <loic@dachary.org>
 # Copyright (C) 2006 Mekensleep <licensing@mekensleep.com>
 #                    24 rue vieille du temple, 75004 Paris
 #
@@ -70,7 +70,8 @@ class PokerLock(threading.Thread):
         try:
             reactor.addSystemEventTrigger('during', 'shutdown', self.stopping)
             while 1:
-                if self.verbose > 2: self.message("loop, queue size " + str(self.q.qsize()))
+                if self.verbose > 2:
+                    self.message("loop, queue size " + str(self.q.qsize()))
                 if not self.running and self.q.empty():
                     self.message("stopped")
                     break
