@@ -1,7 +1,8 @@
 #
 # -*- py-indent-offset: 4; coding: iso-8859-1 -*-
 #
-# Copyright (C) 2008 Loic Dachary <loic@dachary.org>
+# Copyright (C) 2008, 2009 Loic Dachary <loic@dachary.org>
+# Copyright (C) 2009 Johan Euphrosine <proppy@aminche.com>
 # Copyright (C) 2008 Bradley M. Kuhn <bkuhn@ebb.org>
 #
 # This software's license gives you freedom; you can copy, convey,
@@ -104,32 +105,15 @@ def args2packets(args):
 class Request(server.Request):
 
     def getSession(self):
-        self.sitepath = self.args.get('name', [])
-        uid = self.getCookie(self.cookieName())
-        if uid:
-            #
-            # If the uid is -1 it means this uid has been
-            # blacklisted by another session. It triggers
-            # an error, and the uid is removed from the blacklist.
-            # 
-            memcache = self.site.memcache
-            memcache_serial = memcache.get(uid)
-            if memcache_serial == '-1':
-                memcache.delete(uid)
-                raise BlacklistedSession(uid)
-        return server.Request.getSession(self)
-
-    def cookieName(self):
-        return "_".join(['TWISTED_SESSION'] + self.sitepath)
-
-    def expireSessionCookie(self):
-        cookiename = self.cookieName()
-        sessionCookie = self.getCookie(cookiename)
-        if sessionCookie:
-            self.addCookie(cookiename,
-                           sessionCookie,
-                           expires = time.asctime(time.gmtime(seconds() - 3600)) + ' UTC',
-                           path = '/')
+        uid = self.args.get('uid', [self.site._mkuid()])[0]
+        auth = self.args.get('auth', [self.site._mkuid()])[0]
+        explain = self.args.get('explain', ['yes'])[0] == 'yes'
+        try:
+            self.session = self.site.getSession(uid, auth, explain)
+        except KeyError:
+            self.session = self.site.makeSession(uid, auth, explain)
+        self.session.touch()
+        return self.session
 
     def findProxiedIP(self):
         """Return the IP address of the client who submitted this request,
@@ -151,11 +135,15 @@ class Request(server.Request):
 
 class Session(server.Session):
 
-    def __init__(self, site, uid):
+    def __init__(self, site, uid, auth, explain):
         server.Session.__init__(self, site, uid)
+        self.auth = auth
         self.avatar = site.resource.service.createAvatar()
+        self.explain_default = explain
         self.avatar.queuePackets()
-        self.avatar.setExplain(PacketPokerExplain.ALL)
+        self.avatar.setDistributedArgs(uid, auth)
+        if self.explain_default:
+            self.avatar.setExplain(PacketPokerExplain.ALL)
         self.avatar.roles.add(PacketPokerRoles.PLAY)
         self.expired = False
 
@@ -166,6 +154,8 @@ class Session(server.Session):
         self.expired = True
     
     def checkExpired(self):
+        if self.expired:
+            return False
         try:
             #
             # The session may expire as a side effect of the
@@ -174,7 +164,7 @@ class Session(server.Session):
             # is not called : this is intended because the
             # session already expired.
             #
-            self.site.getSession(self.uid)
+            self.site.getSession(self.uid, self.auth, self.explain_default)
             #
             # The session may be replaced by a new session as a side
             # effect of the verifications made by getSession against
@@ -238,7 +228,6 @@ class PokerResource(resource.Resource):
                 request.setResponseCode(http.INTERNAL_SERVER_ERROR)
                 request.setHeader('content-type',"text/html")
                 request.setHeader('content-length', str(len(body)))
-                request.expireSessionCookie()
                 request.write(body)
             if self.verbose >= 0 and request.code != 200:
                 self.error("(%s:%s) " % request.findProxiedIP() + str(body))
@@ -252,24 +241,30 @@ class PokerResource(resource.Resource):
             #
             return True
 
-        deferred.addCallback(lambda result: self.deferRender(request, jsonp, packet))
+        deferred.addCallback(lambda result: self.deferRender(request, jsonp, packet, data))
         deferred.addErrback(pipesFailed)
         return server.NOT_DONE_YET
 
-    def deferRender(self, request, jsonp, packet):
+    def deferRender(self, request, jsonp, packet, data):
         if request.finished:
             #
             # For instance : the request was reverse-proxied to a server.
             #
             return True
         session = request.getSession()
-        d = defer.maybeDeferred(session.avatar.handlePacketDefer, packet)
+        d = defer.maybeDeferred(session.avatar.handleDistributedPacket, request, packet, data)
         def render(packets):
+            if self.verbose > 3:
+                self.message("(%s:%s) " % request.findProxiedIP() + "render " + data + " returns " + str(packets))
             #
             # update the session information if the avatar changed
             #
-            session.site.updateSession(session)
-            session.site.persistSession(session)
+            # *do not* update/expire/persist session if handling
+            # PacketPokerLongPollReturn
+            #
+            if packet.type != PACKET_POKER_LONG_POLL_RETURN:
+                session.site.updateSession(session)
+                session.site.persistSession(session)
             #
             # Format answer
             #
@@ -322,7 +317,6 @@ class PokerImageUpload(resource.Resource):
             request.setResponseCode(http.INTERNAL_SERVER_ERROR)
             request.setHeader('content-type',"text/html")
             request.setHeader('content-length', str(len(body)))
-            request.expireSessionCookie()
             request.write(body)
             request.connectionLost(reason)
             if self.verbose >= 0:
@@ -354,6 +348,29 @@ class PokerImageUpload(resource.Resource):
             request.finish()
             return
 
+class PokerTourneyStartResource(resource.Resource):
+
+    def __init__(self, service):
+        resource.Resource.__init__(self)
+        self.service = service
+        self.verbose = service.verbose
+        self.deferred = defer.succeed(None)
+        self.isLeaf = True
+
+    def message(self, string):
+        print "PokerTourneyStart: " + string
+
+    def render(self, request):
+        if self.verbose > 3:
+            self.message("render " + str(request))
+        tourney_serial = request.args['tourney_serial'][0]
+        self.service.tourneyNotifyStart(int(tourney_serial))
+        body = 'OK'
+        request.setHeader('content-type',"text/html")
+        request.setHeader('content-length', str(len(body)))
+        request.write(body)
+        return True
+
 class PokerAvatarResource(resource.Resource):
 
     def __init__(self, service):
@@ -380,7 +397,6 @@ class PokerAvatarResource(resource.Resource):
             request.setResponseCode(http.INTERNAL_SERVER_ERROR)
             request.setHeader('content-type',"text/html")
             request.setHeader('content-length', str(len(body)))
-            request.expireSessionCookie()
             request.write(body)
             request.connectionLost(reason)
             if self.verbose >= 0:
@@ -407,9 +423,6 @@ class PokerAvatarResource(resource.Resource):
             request.finish()
             return
 
-class BlacklistedSession(Exception):
-    pass
-
 class PokerSite(server.Site):
 
     requestFactory = Request
@@ -438,6 +451,12 @@ class PokerSite(server.Site):
         for path in settings.header.xpathEval("/server/rest_filter"):
             module = imp.load_source("poker_pipe", path.content)
             self.pipes.append(getattr(module, "rest_filter"))
+        resthost = settings.headerGetProperties("/server/resthost")
+        if resthost:
+            resthost = resthost[0]
+            self.resthost = ( resthost['host'], int(resthost['port']), resthost['path'] )
+        else:
+            self.resthost = None
 
     def message(self, string):
         print "PokerSite: " + string
@@ -462,25 +481,21 @@ class PokerSite(server.Site):
             self.sessions[key].expire()
         
     def persistSession(self, session):
-        if len(session.avatar.tables) <= 0 and len(session.avatar.tourneys) <= 0:
+        if len(session.avatar.tables) <= 0 and len(session.avatar.tourneys) <= 0 and (not session.avatar.explain or len(session.avatar.explain.games.getAll()) <= 0):
             session.expire()
+            if self.resthost:
+                self.memcache.delete(session.uid)
             return False
+        is_explain_resthost = self.resthost and session.avatar.explain
+        if is_explain_resthost:
+            session_resthost = self.memcache.get(session.uid)
+            is_new_or_same_resthost = not session_resthost or session_resthost == self.resthost
+            if is_new_or_same_resthost:
+                self.memcache.set(session.uid, self.resthost, time = self.cookieTimeout)
         return True
         
     def updateSession(self, session):
         serial = session.avatar.getSerial()
-        if serial > 0:
-            #
-            # blacklist a session previously associated to the serial
-            #
-            memcache_uid = self.memcache.get(str(serial))
-            if memcache_uid != None and session.uid != memcache_uid:
-                #
-                # the lifespan of this blacklisted uid *must* be greater than
-                # self.sessionTimeout
-                #
-                self.memcache.set(memcache_uid, '-1', time = self.cookieTimeout)
-
         #
         # There is no need to consider the case where session.memcache_serial is
         # zero because nothing needs updating in memcache.
@@ -490,13 +505,7 @@ class PokerSite(server.Site):
                 #
                 # if the user has logged out, unbind the serial that was in memcache
                 #
-                self.memcache.delete(str(session.memcache_serial))
-                self.memcache.replace(session.uid, '0', time = self.cookieTimeout)
-            elif serial != session.memcache_serial:
-                #
-                # if the user changed personality, delete old serial
-                #
-                self.memcache.delete(str(session.memcache_serial))
+                self.memcache.replace(session.auth, '0', time = self.cookieTimeout)
             #
             # for consistency, so that updateSession can be called multiple times without
             # side effects
@@ -509,13 +518,14 @@ class PokerSite(server.Site):
             # because it is how each poker server is informed that
             # a given user is logged in
             #
-            self.memcache.set_multi({ session.uid: str(serial),
-                                      str(serial): session.uid }, time = self.cookieTimeout)
+            self.memcache.set(session.auth, str(serial), time = self.cookieTimeout)
 
-    def getSession(self, uid):
+    def getSession(self, uid, auth, explain):
         if not isinstance(uid, str):
             raise Exception("uid is not str: '%s' %s" % (uid, type(uid)))
-        memcache_serial = self.memcache.get(uid)
+        if not isinstance(auth, str):
+            raise Exception("auth is not str: '%s' %s" % (auth, type(auth)))
+        memcache_serial = self.memcache.get(auth)
         if memcache_serial == None:
             #
             # If the memcache session is gone, trash the current session
@@ -553,20 +563,19 @@ class PokerSite(server.Site):
                 # Create a session with an uid that matches the memcache
                 # key
                 #
-                self.makeSessionFromUid(uid).memcache_serial = memcache_serial
+                self.makeSessionFromUidAuth(uid, auth, explain).memcache_serial = memcache_serial
                 if memcache_serial > 0:
                     self.sessions[uid].avatar.relogin(memcache_serial)
                 
         return self.sessions[uid]
 
-    def makeSessionFromUid(self, uid):
-        session = self.sessions[uid] = self.sessionFactory(self, uid)
+    def makeSessionFromUidAuth(self, uid, auth, explain):
+        session = self.sessions[uid] = self.sessionFactory(self, uid, auth, explain)
         session.startCheckingExpiration(self.sessionCheckTime)
         return session
         
-    def makeSession(self):
-        uid = self._mkuid()
-        session = self.makeSessionFromUid(uid)
+    def makeSession(self, uid, auth, explain):
+        session = self.makeSessionFromUidAuth(uid, auth, explain)
         session.memcache_serial = 0
-        self.memcache.add(uid, str(session.memcache_serial))
+        self.memcache.add(auth, str(session.memcache_serial))
         return session
